/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
 
var show_alerts = true;

$(document).ready(function () {
	$(window).on('beforeunload', function () {
		show_alerts = false;
	});
});

function load_hist(myid) {
	var state = window.history.state;
	if (state !== null && state !== undefined) {
		var data = JSON.parse(state);

		if (data["selectize"] && data[myid]) {
			$('#' + myid).html(data[myid]);
			$('#bz_' + myid + '_input_th').removeClass('bz_default_hidden');
			$('#bz_' + myid + '_input_td').removeClass('bz_default_hidden');
			$('#defined_' + myid).val(1);
			return (1);
		}
	}
	return (0);
}

/*
This plugin reloads ajax values in to selects when hitting the back button
*/
Selectize.define('form_history', function (options) {
	var self = this;

	this.onChange = (function () {
		var original = self.onChange;
		return function () {
			original.apply(this, arguments);
			var myid = self.$input.attr('id');
			var state = window.history.state;
			if (state !== null && state !== undefined) {
				var data = JSON.parse(state);
				data[myid] = $('#' + myid).html();
				history.replaceState(JSON.stringify(data), window.location);
			} else {
				var data = {
					selectize: 1
				};
				data[myid] = $('#' + myid).html();
				history.pushState(JSON.stringify(data), window.location);
			}
		};
	})();
});

Selectize.define('minimum_search_length', function () {
	var self = this;
	var min_length = this.settings.minimum_search_length || null;
	min_length = (!min_length || min_length <= 0) ? 1 : min_length;
	var KEY_TAB = 9;
	var KEY_RETURN = 13;
	this.onKeyUp = (function () {
		var original = self.onKeyUp;
		return function (e) {
			// Ignore tab and enter key to avoid unnecessary check
			if (e.keyCode !== KEY_TAB && e.keyCode !== KEY_RETURN) {
				var value = this.$control_input.val() || '';
				// Make sure it won't close the list if search text is empty because
				// Selectlize will list all the loaded options.
				if (value.length && value.length < min_length) {
					this.close();
					e.preventDefault();
					return;
				}
				// Make sure it reopens the closed list if the new query matches the previous query.
				var last_val = self.lastValue;
				if (last_val.length >= min_length && last_val === value && !self.isOpen) this.open();
			}
			return original.apply(this, arguments);
		};
	})();
});

/**
 * This plugin provides extra keyboard commands for users to navigate the select list.
 */
Selectize.define('extra_keys_control', function () {
	var self = this;
	var KEY_PAGEUP = 33;
	var KEY_PAGEDOWN = 34;
	var KEY_HOME = 36;
	var KEY_END = 35;
	var KEY_A = 65;
	var list_all = this.settings.list_all || function () {};

	this.onKeyDown = (function () {
		var original = self.onKeyDown;
		return function (e) {
			switch (e.keyCode) {
				/*
				Copy from https://github.com/daimor/selectize.js/commit/f0bc92568a4d668a90b5919b8fa98512c396b37d
				This forked commit make changes into selectize.js directly. Instead of doing this, I think it is better
				to write it as a plugin.
				*/
				case KEY_PAGEUP:
					if (self.$activeOption) {
						self.ignoreHover = true;
						var pageSize = Math.round(self.$dropdown_content.innerHeight() / self.$activeOption.outerHeight()) - 1;
						var $prev = self.getAdjacentOption(self.$activeOption, -pageSize);
						if ($prev.length) self.setActiveOption($prev, true, true);
					}
					e.preventDefault();
					return;
				case KEY_PAGEDOWN:
					if (self.$activeOption) {
						self.ignoreHover = true;
						var pageSize = Math.round(self.$dropdown_content.innerHeight() / self.$activeOption.outerHeight()) - 1;
						var $next = self.getAdjacentOption(self.$activeOption, pageSize);
						if ($next.length) self.setActiveOption($next, true, true);
					}
					e.preventDefault();
					return;
				case KEY_HOME:
					if (self.$activeOption) {
						self.ignoreHover = true;
						var $home = this.$dropdown.find('[data-selectable]:first');
						if ($home.length) self.setActiveOption($home, true, true);
						e.preventDefault();
					}
					return;
				case KEY_END:
					if (self.$activeOption) {
						self.ignoreHover = true;
						var $end = this.$dropdown.find('[data-selectable]:last');
						if ($end.length) self.setActiveOption($end, true, true);
						e.preventDefault();
					}
					return;
					/*
					End commit https://github.com/daimor/selectize.js/commit/f0bc92568a4d668a90b5919b8fa98512c396b37d
					*/

					// Load all data when Ctrl + SHIFT + a is pressed.
				case KEY_A:
					if (e.ctrlKey && e.shiftKey) {
						list_all(self);
						e.preventDefault();
					}
					return;
			}
			return original.apply(this, arguments);
		};
	})();
});

/**
 *	If the number of elements not matching in the other array is shorter than the
 *	number of elements in the array then something matches, and vice-versa
 */
function any_matches(array1, array2) {
	return $(array1).not(array2).length != $(array1).length ||
		$(array2).not(array1).length != $(array2).length;
}

/*
This plugin hides fields, and values in fields, base on the values in this field

When product changes, show/hide fields on prod.name in array 
When cf_fu changes, show/hide values in cf_bar, base on cf_fu.selected in array

e.g. Only show value fubar in cf_bar if cf_fu == 'cu fu'

var RELATED_FIELDS = {
    'product': {
		'fields': {
			'cf_fu': ['Prod 1', 'Prod 5', 'Fu Prod'],
			'cf_bar': ['Prod 1', 'Prod 3', 'Bar Prod']
		}
	},	
	cf_fu {
		'values': {
			'cu fu': {
				'cf_bar': ['fubar']
			}
		},
	}
};
*/

Selectize.define('related_fields', function () {
	var self = this;

	this.onChange = (function () {
		var original = self.onChange;
		return function () {
			original.apply(this, arguments);

			var my_id = self.$input.attr('id');
			if (typeof RELATED_FIELDS[my_id] === 'undefined') return;

			var selected_values = $('#' + my_id).get_selected_items();
			if (typeof RELATED_FIELDS[my_id]['fields'] !== 'undefined') {
				$.each(RELATED_FIELDS[my_id]['fields'], function (field, val) {
					/* If none of selected_values are in the this.values */
					if (any_matches(selected_values, val)) {
						$('#' + field).show_field();
					} else {
						$('#' + field).hide_field()
					}
				});
			}
			if (typeof RELATED_FIELDS[my_id]['values'] !== 'undefined') {
				$.each(RELATED_FIELDS[my_id]['values'], function (the_value, fields) {
					if (any_matches(selected_values, [the_value])) {
						$.each(fields, function (the_field, values) {
							$('#' + the_field).show_values(values);
						});
					} else {
						$.each(fields, function (the_field, values) {
							$('#' + the_field).hide_values(values)
						});
					}
				});
			}
		};
	})();
});

Selectize.define('load_from_js', function () {
	var self = this;
	$(this).attr('loaded', false);

	this.onFocus = (function () {
		var original = self.onFocus;
		return function () {
			if ($(this).attr('loaded') == false) {
				var my_id = self.$input.attr('id');

				$('#' + my_id).load_from_js();
				$(this).attr('loaded', true);
			}
			original.apply(this, arguments);
		};
	})();
});

jQuery.fn.extend({
	load_from_js: function () {
		if (!$(this).hasClass('selectized')) return;

		var $sel = $(this).selectize()[0].selectize;
		var my_id = $sel.$input.attr('id');

		if (my_id === 'pool_id') {
			$.each(AGILE_TEAMS, function (team_name, team) {
				if(permitted_teams && ! any_matches(permitted_teams, [team['id']])) {
					return true;
				}
				$sel.addOptionGroup(team['name'], {
					'label': team['name'],
					'value': team['name']
				});
				$.each(team['pools'], function (indexedDB, pool) {
					if (team_name.match('^sst_') && pool['is_sprint'] === 1) {
						return true;
					}
					var data = {};
					data['value'] = pool['id'];
					data['text'] = pool['name'];
					if (default_pool_id && pool['id'] === default_pool_id) {
						data['text'] = pool['name'] + ' *';
					}
					data['optgroup'] = team['name'];
					$sel.addOption(data, team['name']);
				});
			});
			$sel.refreshOptions(true);
		} else if (my_id === 'keywords') {
			$.each(KEYWORDS, function (i, obj) {
				var data = {};
				data['value'] = obj['name'];
				data['text'] = obj['name'];
				data['title'] = obj['title'];
				$sel.addOption(data);
			});
			$sel.refreshOptions(true);
		}
	},
	show_values: function (values) {
		if (!$(this).hasClass('selectized')) return;

		var $sel = $(this).selectize()[0].selectize;
		$.each(values, function (i, value) {
			if ($sel.options[value].disabled === true) {
				$sel.options[value].disabled = false;
				delete $sel.renderCache['option'][value];
			}
		});
		$sel.refreshOptions(false, true);
	},
	hide_values: function (values) {
		if (!$(this).hasClass('selectized')) return;

		var $sel = $(this).selectize()[0].selectize;
		$.each(values, function (i, value) {
			if ($sel.options[value].disabled == false) {
				$sel.options[value].disabled = true;
				delete $sel.renderCache['option'][value];
			}
		});
		$sel.refreshOptions(false, true);
	},
	show_field: function () {
		if ($(this).hasClass('selectized')) {
			var $sel = $(this).selectize()[0].selectize;
			if (($sel).$input.prop('disabled') == true) {
				var myid = $(this).attr('id');
				$sel.enable();
				$('#bz_' + myid + '_input_th').removeClass('bz_default_hidden');
				$("#bz_" + myid + '_input_td').removeClass('bz_default_hidden');
			}
		} else {
			$(this).removeClass('bz_default_hidden');
		}

	},
	hide_field: function () {
		if ($(this).hasClass('selectized')) {
			var $sel = $(this).selectize()[0].selectize;
			if (($sel).$input.prop('disabled') == false) {
				var myid = $(this).attr('id');
				$sel.disable();
				$("#bz_" + myid + '_input_th').addClass('bz_default_hidden');
				$("#bz_" + myid + '_input_td').addClass('bz_default_hidden');
			}
		} else {
			$(this).addClass('bz_default_hidden');
		}
	},
	clearComponentSearch: function (id) {
		var $sel = $(this).selectize()[0].selectize;
		var selected_items = $sel.items;
		$(this).clear_unselected(id);
		$("#component").data('noajax', false);
		if (id) {
			if ($("#product").selectize()[0].selectize.items.length == 0) {
				set_comp_multi(1);
				init_components({
					placehold_text: 'Select a product first'
				});
				$("#component").selectize()[0].selectize.disable();
				$("div.fetch_all").css("display", "none");
			} else {
				set_comp_multi(1);
				init_components({
					select_items: selected_items
				});
				$("#component").selectize()[0].selectize.enable();
				$("div.fetch_all").css("display", "block");
			}
			return;
		}

		var multi = 0;

		if ($("#product").selectize()[0].selectize.items.length == 0) {
			// NOP ATM, might be coder later on
		} else if ($("#product").selectize()[0].selectize.items.length > 1) {
			set_comp_multi(1);
			init_components({
				select_items: selected_items,
				sub_components: show_sub_components
			});
		} else {
			$("#product")._onAjaxSend();
			var rpc = new Rpc('SelectizeJS', 'is_prod_multi', {
					product: $("#product").get_sel_ids()
				})
				.complete(function (cb) {
					$("#product")._onAjaxComplete();
				})
				.fail(function (error) {
					warn(error);
				})
				.done(function (data) {
					set_comp_multi(data.ismulti);
					init_components({
						select_items: selected_items,
						sub_components: show_sub_components
					});
				});
		}
	},
	clearSel: function (id) {
		if (!$(this).hasClass('selectized')) {
			return;
		}
		var $selHandle = $(this).selectize()[0].selectize;
		$selHandle.clearOptions();
		$selHandle.lastQuery = null;
		$selHandle.loadedSearches = {};
		$selHandle.currentResults = null;
		$selHandle.refreshOptions(false);
	}

});

/*
NOTE: This method is related to 'js/field-ajax.js' which might be deleted in Bugzilla 5.0 so we
will need to review this code during the Bugzilla 5.0 upgrades for other solution.

The purpose of this method is to make the ajax request to exclude unnecessary component field
in show_bug.cgi page when the product field changes. Fetching components could be expensive for
some products, such as Fedora.
*/
function getBzFieldList(exclude_fields) {
	return $.grep(bz_field_list, function (field) {
		if ($.inArray(field, exclude_fields) === -1) return true;
		return false;
	});
}


function warn(error) {
	if (show_alerts) {
		$('#message').append('<div id="errormsg">' + error.message + '</div>');
		$("#errormsg").dialog({
			buttons: {
				OK: function () {
					$(this).dialog("close");
				}
			},
			title: "RPC Error",
		});
	}
}


function set_comp_multi(multi) {
	var mytest = $('#component').attr('multiple');
	if (multi != mytest) {
		var $select = $('#component');
		var attributes = $select.prop("attributes");
		$('#bz_component_input').html("");
		$select = $('<select></select>');
		$.each(attributes, function () {
			if (this.name == 'onchange') {
				return;
			}
			$select.attr(this.name, this.value);
		});

		if (multi != "0") {
			$select.attr('multiple', true);
		} else {
			$select.attr('multiple', false);
		}
		$('#bz_component_input').append($select);
		$('#bz_component_input').append('<div class="fetch_all" onclick="$(\'#component\').fetch_all()"><i class="fa fa-download"></i><span title="Click to list all components">Click to list all components</span></div>');

	}
}

function _value_id(field_name, id) {
	return 'v' + id + '_' + field_name;
}

jQuery.fn.extend({
	update_displayed_values: function (target_field_id, rules) {
		var $sel = $(this).selectize()[0].selectize;
		if (!$('#' + target_field_id).hasClass('selectized')) return;

		var $target = $('#' + target_field_id).selectize()[0].selectize;
		var my_name = $(this).attr('id');
		var selected_id = $sel.options[$sel.items[0]].id;

		for (var key in $target.options) {
			var show = 1;
			var opt = $target.options[key];
			$.each(rules, function (id, vals) {
				id = _value_id(my_name, id);
				// not not is true
				if (id !== selected_id) {
					$.each(vals, function (opt_idx, opt_id) {
						opt_id = _value_id(target_field_id, opt_id);
						if (opt.id == opt_id) {
							show = 0;
						}
					});
				}
			});

			if (show) {
				if ($target.options[key].disabled == true) {
					$target.options[key].disabled = false;
					delete $target.renderCache['option'][key];
				}
			} else {
				if ($target.options[key].disabled == false) {
					$target.options[key].disabled = true;
					delete $target.renderCache['option'][key];
				}
			}
		};

		$target.refreshOptions(false, true);

		return;
	},
	select_first: function () {
		var $sel;
		var $selize = $(this).selectize();
		if ($selize[0] !== undefined) {
			$sel = $selize[0].selectize;
		} else {
			$sel = $selize.selectize;
		}
		var lowest = Number.MAX_SAFE_INTEGER;
		var found;
		for (var key in $sel.options) {
			if ($sel.options[key].$order < lowest) {
				lowest = $sel.options[key].$order
				found = key
			}
		}
		if (typeof found !== 'undefined') {
			$sel.setValue($sel.options[found].value);
		}
	},
	// Set description for component
	set_descr: function () {
		var $sel = $(this).selectize()[0].selectize;
		var text = '';
		$(this).find('option').each(function () {
			if ($(this).is(":selected") && $(this).val().length) {
				var $obj = $sel.options[$(this).val()];
				text = text + $obj['title'] + "<br>";
			}
		});
		$("#comp_desc").html(text);
	},
	// Fetch all options for this select
	fetch_all: function (quiet, force, select_items) {
		var $sel = $(this).selectize()[0].selectize;
		var query = '..';
		// Don't fetch all again if we already did
		if (force || $sel.$input.data('noajax') === undefined || !$sel.$input.data('noajax')) {
			$sel.$input.data('noajax', false);
			$.each($sel.options, function (myval) {
				if (!$sel.items || $sel.items.indexOf(myval) === -1) {
					$sel.removeOption(myval);
				}
			});
			$sel.select_items = select_items;
			$sel.lastQuery = null;
			$sel.loadedSearches = {};
			$sel.currentResults = null;
			$sel.onSearchChange(query);
			$sel.unlock();
			$sel.enable();
		}
		if (!quiet) $sel.open();
	},
	// Get the ID of all selected items
	get_sel_ids: function () {
		var $sel = $(this).selectize()[0].selectize;
		return $.map($sel.items, function (value) {
			return $sel.options[value]['id']
		}).join(',');
	},
	get_selected_items: function () {
		var $sel = $(this).selectize()[0].selectize;

		var texts = [];

		$.each($sel.items, function (idx, value) {
			texts.push(value);
		});
		return texts;
	},
	get_selected_item: function () {
		var $sel = $(this).selectize()[0].selectize;

		var text = '';

		if ($sel.items[0]) {
			text = $sel.items[0];
		}

		return text;
	},
	get_selected_item_text: function () {
		var $sel = $(this).selectize()[0].selectize;

		var text = '';

		if ($sel.items[0]) {
			text = $sel.getItem($sel.items[0]).text().trim();
		}

		return text;
	},
	clear_unselected: function (id) {
		var $selize = $(this).selectize();
		if ($selize[0] !== undefined) {
			$sel = $selize[0].selectize;
		} else {
			$sel = $selize.selectize;
		}
		$.each($sel.options, function (myval) {
			if ((id && $('#' + id).selectize()[0].selectize.items.length == 0) || $sel.items.indexOf(myval) === -1) {
				$sel.removeOption(myval);
			}
		});

		$(this).find('optgroup:empty').each(function () {
			$sel.removeOptionGroup($(this).value);
		});
		$sel.lastQuery = null;
		$sel.loadedSearches = {};
		$sel.currentResults = null;
		$sel.refreshOptions(false);
	},
	update_select: function (fields, id, rpc_comp, rpc_func, rpc_id_field, inactive) {
		var $selize;
		// This is required because the forms sometimes have text instead of selects.
		if (!$(this).hasClass('selectized')) {
			$(this).html('<select id="' + $(this).attr('id') + '"><option>---</option></select>');
			$selize = $(this).selectize({
				selectOnTab: true
			});
		} else {
			$selize = $(this).selectize();
		}

		if ($selize === Array) {
			$sel = $selize[0].selectize;
		} else {
			$sel = $selize.selectize;
		}
		var $me = $(this);
		var rpc;
		var data = {};
		rpc_id_field = rpc_id_field || 'names';

		if ($("#" + id).selectize()[0].selectize.items.length == 0) {
			$me.clear_unselected();
			if ($sel.items.length == 0) {
				$sel.settings.placeholder = 'Select a ' + id + ' first';
				$sel.updatePlaceholder();
				$sel.disable();
				return
			}
		}

		data[rpc_id_field] = $("#" + id).selectize()[0].selectize.getValue();
		data['include_fields'] = fields;
		$me._onAjaxSend();
		return rpc = new Rpc(rpc_comp, rpc_func, data)
			.complete(function (cb) {
				$me._onAjaxComplete();
			})
			.fail(function (error) {
				warn(error);
			})
			.done(function (res) {
				var data = [];
				var field = fields[0];
				$me.clear_unselected();
				var count = 0;
				if (res[id + "s"] === undefined) return;
				for (i = 0, n = res[id + "s"].length; i < n; i++) {
					var outer = res[id + "s"][i];
					if (outer[field] === undefined) continue;

					for (j = 0, m = outer[field].length; j < m; j++) {
						var inner = outer[field][j];
						if (!inactive && inner.is_active === false) continue;
						data[count] = {
							text: inner.name,
							value: inner.name,
							id: "v" + inner.id + "_" + id
						};
						count = count + 1;
					}
				}

				$sel.addOption(data);
				$sel.refreshOptions(false);
				if (count > 1) {
					$sel.enable();
				} else {
					$sel.setValue(data[0].value);
					$sel.disable();
				}
				$sel.settings.placeholder = ' ';
				$sel.updatePlaceholder();
			});
	},
	reset_select: function (id, rpc_comp, rpc_func) {
		var $sel = $(this).selectize()[0].selectize;
		var $me = $(this);
		var rpc;
		var data = {};

		$me.clearSel();

		data['id'] = $("#" + id).selectize()[0].selectize.getValue();
		$("#" + id)._onAjaxSend();
		return rpc = new Rpc(rpc_comp, rpc_func, data)
			.complete(function (cb) {
				$("#" + id)._onAjaxComplete();
			})
			.fail(function (error) {
				warn(error);
			})
			.done(function (res) {
				var data = [];
				var count = 0;
				for (i = 0, n = res.length; i < n; i++) {
					var rec = res[i];
					data[count] = {
						text: rec[1],
						value: rec[0]
					};
					count = count + 1;
				}

				$sel.addOption(data);
				$sel.refreshOptions(false);
				$sel.enable();
				$sel.settings.placeholder = ' ';
				$sel.updatePlaceholder();
			});
	},
	// Can be called on selects (selectized) or text boxes
	_onAjaxSend: function () {
		var node = this.parent();
		if (this[0] && this[0].selectize) {
			node = this[0].selectize.$control_input.parent();
		}

		if (!node.find('.fa-spinner').length) {
			node.append('<i class="fa fa-spinner fa-pulse"></i>');
		}
	},
	// Can be called on selects (selectized) or text boxes
	_onAjaxComplete: function () {
		var node = this.parent();
		node.find('.fa-spinner').remove();
	}

});

function init_components(args) {
	args = args || {};
	args.mywidth = args.mywidth || "25em";
	args.placehold_text = args.placehold_text || "Type a component name. Ctrl+Shift+A to list all";
	args.disabled = args.disabled || 0;
	args.sub_components = args.sub_components || null;
	args.id = args.id || 'component';
	args.product_id = args.product_id || 'product';
	args.follow_product = args.follow_product || 0;
	load_hist(args.id);
	args.product = args.product || null;

	var foo;

	$("#" + args.id).attr('placeholder', args.placehold_text);
	var $sel = $("#" + args.id).selectize({
		sortField: {
			field: 'text',
			direction: 'asc'
		},
		loadThrottle: 1000,
		selectOnTab: true,
		maxOptions: null,
		// Set this to false because it will open the dropdown list even if you just tab pass this element
		// which I think is quite annoying. The native selectbox won't expand the list when focus. User
		// can either click the caret or hit the down arrow key to open the dropdown.
		openOnFocus: false,
		onInitialize: function () {
			fetch_all_listener(this, args);
		},
		// Provides a callback function to fetch all list when shortcut key is pressed.
		list_all: function () {
			$("#" + args.id).fetch_all();
		},
		plugins: ['remove_button', 'minimum_search_length', 'extra_keys_control', 'form_history', 'related_fields'],
		render: {
			option: function (item, escape) {
				return '<div title="' + escape(item.title || "") + '">' +
					'<span>' + escape(item.value) + '</span>' +
					'</div>';
			}
		},
		load: function (query, callback) {
			var $sel = this;
			$sel.$input.data('noajax', args.noajax || $sel.$input.data('noajax') || false);
			args.noajax = false;
			if (!query.length) return callback();
			if ($sel.$input.data('noajax')) return callback();
			if (args.auto) $sel.$input.data('noajax', true);

			var data = {};
			data['term'] = query;
			data['descrs'] = args.descrs;
			data['disabled'] = args.disabled;
			data['product'] = args.product || $("input[name=product]").val() || $("#" + args.product_id).get_sel_ids();
			if (!data['product'].length) return callback();
			$("#" + args.id)._onAjaxSend();
			return foo = new Rpc('SelectizeJS', 'list_components', data)
				.complete(function () {
					$("#" + args.id)._onAjaxComplete();
				})
				.fail(function (error) {
					warn(error);
				})
				.done(function (res) {
					// Don't do rpc again if we got the full list.
					if (query === '..') $sel.$input.data('noajax', true);
					callback(res);
				});
		}
	})[0].selectize;

	$("#container_component .selectize-control input").attr('placeholder', args.placehold_text);
	if (args.auto) {
		$("#" + args.id).fetch_all(1);
	}

	var select_items = args.select_items || [];
	if (!args.auto && select_items.length) {
		var params = {};
		params['components'] = select_items
		params['product'] = args.product || $("input[name=product]").val() || $("#" + args.product_id).get_sel_ids();
		if (params['product'] && params['product'].length) {
			$sel.clearOptions();
			$("#" + args.id)._onAjaxSend();
			new Rpc('SelectizeJS', 'list_components', params)
				.complete(function () {
					$("#" + args.id)._onAjaxComplete();
				})
				.fail(function (error) {
					warn(error);
				})
				.done(function (res) {
					$.each(res, function (idx, data) {
						$sel.addOption(data);
						$sel.addItem(data['value']);
					});
					if (args.sub_components) {
						args.sub_components();
						$("#" + args.id).on('change', function () {
							args.sub_components();
							assign_to_default();
						});
					}
				});
		}
		return;
	} else if (select_items.length) {
		$sel.items = [];
		$sel.addItems(select_items);
	}
	if (args.sub_components) args.sub_components();
	if (args.auto) {
		$("#" + args.id).fetch_all(true);
	}
	if (args.follow_product) {
		$('#' + args.product_id).on('change', function () {
			$('#' + args.id).clearComponentSearch(args.product_id);
		});

	}
}

function init_products(args) {
	args = args || {};
	args.mywidth = args.mywidth || "25em";
	args.placehold_text = args.placehold_text || "";
	args.disabled = args.disabled || 0;
	args.id = args.id || 'product';
	args.classification_id = args.classification_id || 'classification';
	args.classification_id = '#' + args.classification_id;
	var foo;
	load_hist(args.id);

	if (args.def) $("#" + args.id + " option:selected").remove();

	$("#" + args.id).attr('placeholder', args.placehold_text);
	var $sel = $("#" + args.id).selectize({
		sortField: {
			field: 'text',
			direction: 'asc'
		},
		loadThrottle: 1000,
		selectOnTab: true,
		maxOptions: null,
		openOnFocus: false,
		optgroupField: "classification",
		optgroupValueField: "name",
		optgroupLabelField: "description",
		lockOptgroupOrder: true,
		optgroups: c_optgroups,
		onInitialize: function () {
			fetch_all_listener(this, args);
		},
		plugins: ['remove_button', 'extra_keys_control', 'related_fields'],
		render: {
			option: function (item, escape) {
				return '<div class="option" title="' + escape(item.title) + '" id="' + escape(item.id) + '" data-value="' + escape(item.value) + '">' +
					'<span>' + escape(item.text) + '</span>' +
					'</div>';
			}
		},
		load: function (query, callback) {
			var $sel = this;
			$sel.$input.data('noajax', args.noajax || $sel.$input.data('noajax') || false);
			args.noajax = false;
			if (!query.length) return callback();
			if ($sel.$input.data('noajax')) return callback();
			if (args.auto) $sel.$input.data('noajax', true);
			if (args.onclick) $sel.$input.data('noajax', true);

			var data = {};
			data['term'] = query;
			data['descrs'] = args.descrs;
			data['disabled'] = args.disabled;
			if (args.classification) {
				data['classification'] = $(args.classification_id).selectize()[0].selectize.getValue();
			}
			$("#" + args.id)._onAjaxSend();
			return foo = new Rpc('SelectizeJS', 'list_products', data)
				.complete(function (cb) {
					$("#" + args.id)._onAjaxComplete();
					if (args.def) $sel.setValue(args.def);
					if (args.callback) args.callback();
				})
				.fail(function (error) {
					warn(error);
				})
				.done(function (res) {
					// Don't do rpc again if we got the full list.
					if (query === '..') $sel.$input.data('noajax', true);
					callback(res);
				});
		}
	})[0].selectize;
	$("#field_container_product .selectize-control input").attr('placeholder', args.placehold_text);
	if (args.auto) {
		$("#" + args.id).fetch_all(true);
	}
	$("#" + args.id).selectize()[0].selectize.close();
	if (args.classification) {
		$(args.classification_id).on('change', function () {
			var select_items = get_selected_items_text($(args.classification_id).selectize()[0].selectize);
			$("#" + args.id).fetch_all(true, true, select_items);
		});
	}
}

function init_sub_components(args) {
	args = args || {};
	args.mywidth = args.mywidth || "25em";
	args.placehold_text = args.placehold_text || "Type a sub-component name";
	args.disabled = args.disabled || 0;
	args.valueField = args.valueField || 'value';
	args.use_name = args.use_name || 0;
	args.hook_component = args.hook_component || 0;

	var foo;
	var myid = args.myid || 'rh_sub_component';
	var show = load_hist(myid);

	args.nohide = args.nohide || show || 0;

	$("#" + myid).attr('placeholder', args.placehold_text);
	$("#" + myid).selectize({
		valueField: args.valueField,
		loadThrottle: 1000,
		selectOnTab: true,
		maxOptions: null,
		openOnFocus: false,
		onInitialize: function () {
			fetch_all_listener(this, args);
		},
		plugins: ['remove_button', 'minimum_search_length', 'extra_keys_control', 'form_history', 'related_fields'],
		render: {
			option: function (item, escape) {
				return '<div title="' + escape(item.title || "") + '">' +
					'<span>' + Array(3 * (item.level || 0)).join("&nbsp;") + escape(item.text) + '</span>' +
					'</div>';
			}
		},
		onOptionAdd: function (value, data) {
			if (data['selected']) this.addItem(value);
		},
		load: function (query, callback) {
			var $sel = this;
			if (!query.length) return callback();
			$sel.$input.data('noajax', args.noajax || $sel.$input.data('noajax') || false);
			args.noajax = false;
			if ($sel.$input.data('noajax')) return callback();
			if (args.auto) $sel.$input.data('noajax');

			var data = {};
			data['term'] = query;
			if (this.select_items) data['select'] = this.select_items.slice();
			this.select_items = null;
			data['product'] = args.product || $("input[name=product]").val() || $("#product").get_sel_ids();
			data['component'] = $("#component").selectize()[0].selectize.getValue();
			data['descrs'] = args.descrs;
			data['disabled'] = args.disabled;
			if (!data['product'].length) return callback();
			if (!data['component'].length) return callback();
			// On some pages we stick the ID in another field
			if (!/^v\d+/.test(data['product'])) data['product'] = $("input[name=product_id]").val();
			if (!/^v\d+/.test(data['product'])) return callback();
			if (args.use_name) data['use_name'] = 1;
			$("#" + myid)._onAjaxSend();
			return foo = new Rpc('SelectizeJS', 'list_subcomponents', data)
				.complete(function () {
					$("#" + myid)._onAjaxComplete();
					if (args.def) $sel.addItem(args.def);
				})
				.fail(function (error) {
					warn(error);
				})
				.done(function (data) {
					if (data[0] == undefined) {
						if (args.nohide === 0) hide_and_clear_sub_components(myid);
					} else {
						$('#bz_' + myid + '_input_th').removeClass('bz_default_hidden');
						$('#bz_' + myid + '_input_td').removeClass('bz_default_hidden');
						$('#defined_' + myid).val(1);
						$('#container_' + myid + ' .selectize-control input').attr('placeholder', args.placehold_text);
						callback(data);
					}
				});
		}
	});
	$('#container_' + myid + ' .selectize-control input').attr('placeholder', args.placehold_text);
	if (args.auto) {
		$("#" + myid).fetch_all(1);
	}

	if (args.hook_component) {
		$("#component").on('change', function () {
			show_sub_components(myid);
		});
	}
}

function show_sub_components(myid) {
	if (myid === undefined) myid = "rh_sub_component";

	var $sub_comp_sel = $("#" + myid).selectize()[0].selectize;
	var product = $("input[name=product]").val() || $("#product").get_sel_ids();
	var component = $("#component").selectize()[0].selectize.getValue();
	var select_items = get_selected_items_text($sub_comp_sel);
	hide_and_clear_sub_components(myid);
	if (component === "") return;
	$("#" + myid).fetch_all(1, true, select_items);

}

function get_selected_items_text($sel) {
	var texts = [];
	$.each($sel.items, function (idx, value) {
		//		texts.push($sel.getItem(value).text().trim());
		texts.push(value.trim());
	});
	return texts;
}

function hide_and_clear_sub_components(myid) {
	$("#bz_" + myid + '_input_th').addClass('bz_default_hidden');
	$("#bz_" + myid + '_input_td').addClass('bz_default_hidden');
	$('#defined_rh_sub_component').val(0);
	$("#" + myid).clearSel();
	$("#" + myid).selectize()[0].selectize.disable();
}

function fetch_all_listener($sel, args) {
	var KEY_DOWN = 40;
	$(document).on('mousedown.bz', function (e) {
		// Open or fetch all data when user clicks the selectize control.
		if (e.target === $sel.$control[0] && !$sel.isFocused) {
			fetch_all_or_open($sel, args.onclick);
		}
	});
	$sel.$control_input.on('keydown.bz', function (e) {
		// Open or fetch all data when user hit the down arrow key.
		if (e.keyCode === KEY_DOWN) {
			fetch_all_or_open($sel, args.onclick);
		}
	});
}

function fetch_all_or_open($sel, fetch) {
	if (fetch) {
		$sel.$input.fetch_all()
	} else if (!$sel.isOpen) {
		$sel.open();
	}
}

function assign_to_default() {
	var fields = ['set_default_assignee', 'set_default_qa_contact', 'set_default_docs_contact', 'set_default_pool'];
	for (var i in fields) {
		if ($("#" + fields[i])) {
			$("#" + fields[i]).attr('checked', true);
			$("#" + fields[i]).parent().removeClass('bz_default_hidden');
			$("#" + fields[i] + "_label").css("font-weight", "bold");
		}
	}
}


function reassign_to_default(subcomp) {
	var product = $("input[name=product]").val() || $("#product").selectize()[0].selectize.getValue();
	var compSelHandle = $("#component").selectize()[0].selectize;

	var component = compSelHandle.getValue();
	if (!product || component === "" || component.length === 0) return;

	var subcomponent;
	if (subcomp) {
		subcomponent = $("#" + subcomp + " option:selected").val();
	}
	var fields = ['assignee', 'qa_contact', 'docs_contact', 'initial_cc'];
	for (var id in fields) {
		$('#' + fields[id])._onAjaxSend();
	}

	var rpc = new Rpc('SelectizeJS', 'list_roles', {
			product: product,
			component: component,
			subcomponent: subcomponent
		})
		.fail(function (error) {
			warn(error);
		})
		.done(function (data) {
			for (var id in fields) {
				$('#' + fields[id])._onAjaxComplete();
			}

			if (data == null || data[0] == undefined) {
				return;
			}
			for (var i in fields) {
				if ($("#" + fields[i])) {
					var id = "#" + fields[i];
					if (i == 0) {
						id = '#assigned_to';
					}
					var cur_val = $(id).val();
					var new_val = data[0][fields[i]];

					if(new_val === undefined){
						$(id).val('');
						$(id).text('');
						continue;
					}

					if (cur_val == new_val) {
						continue;
					}

					if (id == '#initial_cc') {
						$(id).text(new_val);
					} else {
						$(id).val(new_val['login']);
						if (new_val['insider']) {
							$(id).addClass('redhat_user');
						} else {
							$(id).removeClass('redhat_user');
						}
					}

				}
			}

			$("#flags tbody.bz_flag_type").each(function () {
				var not_found = 1;
				var $sel = $(this).find('select');
				for (var j in data[0]['flags']['bug']) {
					if ($sel.attr('id') == 'flag_type-' + data[0]['flags']['bug'][j]['id']) {
						if (data[0]['flags']['bug'][j]['is_requestable'] == 1) {
							$(this).removeClass('bz_default_hidden');
							not_found = 0;
						}
						break;
					}
				}
				if (not_found) {
					$(this).addClass('bz_default_hidden');
				}
			});
		});
}

function update_selects(empty, inactive) {
	inactive = inactive ? true : false;
	if (empty) {
		$('#version').clearSel();
		$('#target_milestone').clearSel();
		if ($('#target_release').length > 0) {
			$('#target_release').clearSel();
		}
	}
	$('#version').update_select(['versions'], 'product', 'Product', 'get', 'names', inactive);
	$('#target_milestone').update_select(['milestones'], 'product', 'Product', 'get', 'names', inactive);
	if ($('#target_release').length > 0) {
		$('#target_release').update_select(['releases'], 'product', 'Product', 'get', 'names', inactive);
	}
}