/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 */
 
var keys = {
  ENTER: 13,
  ESC: 27,
  F1: 112,
  F12: 123,
  LEFT: 37,
  RIGHT: 39,
  TAB: 9
};

alertify.defaults.notifier.position = 'top-right';
alertify.defaults.transitionOff = true;
alertify.defaults.modal = false;

if (!alertify.editCommentBox) {
  alertify.dialog('editCommentBox', function factory() {
    var input = $('<textarea rows="15" cols="50"></textarea>');
    var sec_label = $('<label>Lock history: </label>');
    var sec_input = $('<input type="checkbox"></input>');
    var comment;
    var c_count;
    return {
      main: function (text_elem, comment_id, comment_count, message, allow_secure) {
        comment = text_elem;
        this.comment_id = comment_id;
        c_count = comment_count;
        this.message = message;
        this.set('title', 'Edit comment ' + comment_count);
        this.secure = allow_secure;
      },
      setup: function () {
        return {
          buttons: [{
              text: alertify.defaults.glossary.ok,
              className: alertify.defaults.theme.ok,
            },
            {
              text: alertify.defaults.glossary.cancel,
              key: keys.ESC,
              invokeOnClose: true,
              className: alertify.defaults.theme.cancel,
            }
          ],
          focus: {
            element: null,
            select: false
          },
        };
      },
      build: function () {
        input.className = alertify.defaults.theme.input;
        this.elements.content.appendChild(input[0]);
      },
      prepare: function () {
        $(input).val(this.message);
        if (this.secure) {
          this.elements.content.appendChild(sec_label[0]);
          this.elements.content.appendChild(sec_input[0]);
        }
      },
      callback: function (closeEvent) {
        var returnValue;
        switch (closeEvent.index) {
          case 0:
            var new_val = input[0].value;
            if (new_val !== this.message) {
              $(input)._onAjaxSend();
              // RPC to update message
              var rpc = new Rpc('RedHat', 'edit_comment', {
                  comment: this.comment_id,
                  text: new_val,
                  render: 1,
                  security: $(sec_input).prop("checked") ? 1 : 0,
                  new_token: 1
                })
                .complete(function () {
                  $(input)._onAjaxComplete();
                })
                .fail(function (error) {
                  warn(error);
                })
                .done(function (res) {
                  old_ts = res['delta_ts'];
                  old_time = new Date(old_ts).getTime()
                  $('[name="delta_ts"]').val(old_ts);
                  $('[name="token"]').val(res['new_token']);
                  $(comment).html(res['html']);
                  $('#comment_history_' + c_count).removeClass('bz_default_hidden');
                  $('#comment_activity_count_' + c_count).html(res['activity_count']);
                });
            } else {
              alertify.warning("No changes detected!");
            }
            break;
          case 1:
            // NOP
            break;
        }
        $(input).remove();
      }
    };
  }, true);
}

function edit_comment(comment_id, comment_count, allow_secure) {
  var text_elem = document.getElementById('comment_text_' + comment_count);
  var text = getText(text_elem);
  alertify.editCommentBox(text_elem, comment_id, comment_count, text, allow_secure).resizeTo('80%', '70%');
}